/* David Leonard, 2002. Public domain. */
/* $Id: hint.c,v 1.1 2002/02/16 13:01:31 d Exp $ */

/*
 * Provide a hint of what cell to look at next.
 */

#include <stdio.h>
#include "mine.h"
#include "hint.h"

/* 
 * Provide a hint by finding a cell that can be exposed or flagged.
 * (Trusts the state of flags already set)
 * Return 1 if one is found, 0 if not
 */
int
hint(m, rowp, colp)
	struct mine *m;
	int *rowp, *colp;
{
	int rows, cols;
	int row, col, i;

	mine_size(m, &rows, &cols, NULL);
	for (row = 0; row < rows; row++)
	for (col = 0; col < cols; col++)
	{
		int flagged, surrounding, unexposed;

		if (mine_status(m, row, col) != STATUS_TESTED)
			continue;
		surrounding = mine_test(m, row, col);

		/* count the number of unexposed surrounding cells */
		for (flagged = unexposed = i = 0; i < 8; i++) {
			int r = row + delta[i].dx;
			int c = col + delta[i].dy;
			int status;

			if (r >= 0 && r < rows && c >=0 && c < cols) {
			    status = mine_status(m, r, c);
			    if (status == STATUS_MARKED_FLAG)
				flagged++;
			    if (status != STATUS_TESTED)
				unexposed++;
			} 
		}

		if (flagged > surrounding) {
			*rowp = row;
			*colp = col;
			return HINT_TOOMANY;
		}

		if (flagged == surrounding && flagged < unexposed) {
			/* all the unflagged, unexposed cells are exposable */
			for (i = 0; i < 8; i++) {
				int r = row + delta[i].dx;
				int c = col + delta[i].dy;
				if (r >= 0 && r < rows && c >=0 && c < cols
				    && mine_status(m, r, c) == STATUS_UNTESTED)
				{
					*rowp = r;
					*colp = c;
					return HINT_EXPOSE;
				}
			}
			/* notreached? */
		}

		if (unexposed == surrounding && flagged < surrounding) {
			/* all the unflagged, unexposed cells are flaggable */
			for (i = 0; i < 8; i++) {
				int r = row + delta[i].dx;
				int c = col + delta[i].dy;
				if (r >= 0 && r < rows && c >=0 && c < cols
				    && mine_status(m, r, c) == STATUS_UNTESTED)
				{
					*rowp = r;
					*colp = c;
					return HINT_FLAG;
				}
			}
			/* notreached? */
		}
	}
	return HINT_FAILED;
}
