/* David Leonard, 2002. Public domain. */
/* $Id: hiscore.c,v 1.4 2002/02/16 13:57:51 d Exp $ */

/*
 * Hi-score recording
 */

#include <db.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/param.h>

#include "hiscore.h"

#define PATH_SCOREFILE	"/var/games/cmine.scores"

struct scorekey {
	int rows, cols, mines;
};

struct scorevalue {
	int time;
	char login[MAXLOGNAME];
	struct timeval when;
};

/* Record a score if it is the highest. Return 1 if it was stored. */
int
hiscore(rows, cols, mines, time)
	int rows, cols, mines, time;
{
	DB *db;
	DBT key, data;
	struct scorekey scorekey;
	struct scorevalue scorevalue;
	char *login;

	login = getlogin();
	if (login == NULL)
		login = "?";
	db = dbopen(PATH_SCOREFILE, O_CREAT|O_RDWR|O_EXLOCK, 0660, 
	    DB_HASH, NULL);
	if (!db)
		return 0;
	/* Read the best score so far */
	scorekey.rows = rows;
	scorekey.cols = cols;
	scorekey.mines = mines;
	key.data = &scorekey;
	key.size = sizeof scorekey;
	data.data = NULL;
	data.size = 0;
	/* Don't bother if we aren't better than an existing */
	if ((*db->get)(db, &key, &data, 0) == 0)
		if (((struct scorevalue *)data.data)->time >= time)
			return 0;
	/* Save the better score */
	scorevalue.time = time;
	gettimeofday(&scorevalue.when, NULL);
	strlcat(scorevalue.login, login, sizeof scorevalue.login);
	data.data = &scorevalue;
	data.size = sizeof scorevalue;
	(*db->put)(db, &key, &data, 0);
	(*db->close)(db);
	return 1;
}

/* Print all the hi scores */
void
showscores()
{
	DB *db;
	DBT key, data;
	struct scorekey scorekey;
	struct scorevalue scorevalue;
	int flag;

	db = dbopen(PATH_SCOREFILE, O_RDONLY|O_SHLOCK, 0660, DB_HASH, NULL);
	if (!db) {
		perror(PATH_SCOREFILE);
		return;
	}
	printf("%4s %4s %4s  %4s %-.*s\n",
		"Rows", "Cols", "Mine",  "Time", 
		(int) sizeof scorevalue.login, "Login");
	/* Read the best score so far */
	key.data = &scorekey;
	key.size = sizeof scorekey;
	data.data = &scorevalue;
	data.size = sizeof scorevalue;
	flag = R_FIRST;
	while ((*db->seq)(db, &key, &data, flag) == 0) {
		struct scorevalue *v = (struct scorevalue *)data.data;
		struct scorekey *k = (struct scorekey *)key.data;
		printf("%-4d %-4d %-4d  %-4d %-.*s\n",
			k->rows, k->cols, k->mines,
			v->time,
			sizeof v->login, v->login);
			
		flag = R_NEXT;
	}
	(*db->close)(db);
}

